@@ -2,6 +2,7 @@
 
 import logging
 import os
+import re
 import shutil
 import sys
 
@@ -16,10 +17,10 @@ VISIBILITY_CHOICES = ['public', 'unlisted', 'private', 'direct']
 
 def language(value):
     """Validates the language parameter"""
-    if len(value) != 3:
+    if len(value) != 2:
         raise ArgumentTypeError(
-            "Invalid language specified: '{}'. Expected a 3 letter "
-            "abbreviation according to ISO 639-2 standard.".format(value)
+            "Invalid language. Expected a 2 letter abbreviation according to "
+            "the ISO 639-1 standard."
         )
 
     return value
@@ -40,6 +41,42 @@ def timeline_count(value):
     return n
 
 
+DURATION_UNITS = {
+    "m": 60,
+    "h": 60 * 60,
+    "d": 60 * 60 * 24,
+}
+
+
+def duration(value: str):
+    match = re.match(r"""^
+        (([0-9]+)\s*(days|day|d))?\s*
+        (([0-9]+)\s*(hours|hour|h))?\s*
+        (([0-9]+)\s*(minutes|minute|m))?\s*
+        (([0-9]+)\s*(seconds|second|s))?\s*
+    $""", value, re.X)
+
+    if not match:
+        raise ArgumentTypeError(f"Invalid duration: {value}")
+
+    days = match.group(2)
+    hours = match.group(5)
+    minutes = match.group(8)
+    seconds = match.group(11)
+
+    days = int(match.group(2) or 0) * 60 * 60 * 24
+    hours = int(match.group(5) or 0) * 60 * 60
+    minutes = int(match.group(8) or 0) * 60
+    seconds = int(match.group(11) or 0)
+
+    duration = days + hours + minutes + seconds
+
+    if duration == 0:
+        raise ArgumentTypeError("Empty duration")
+
+    return duration
+
+
 def editor(value):
     if not value:
         raise ArgumentTypeError(
@@ -337,6 +374,14 @@ POST_COMMANDS = [
                 "help": "ISO 8601 Datetime at which to schedule a status. Must "
                         "be at least 5 minutes in the future.",
             }),
+            (["--scheduled-in"], {
+                "type": duration,
+                "help": """Schedule the toot to be posted after a given amount
+                        of time. Examples: "1 day", "2 hours 30 minutes",
+                        "5 minutes 30 seconds" or any combination of above.
+                        Shorthand: "1d", "2h30m", "5m30s". Must be at least 5
+                        minutes.""",
+            }),
             (["-t", "--content-type"], {
                 "type": str,
                 "help": "MIME type for the status text (not supported on all instances)",
