@@ -13,39 +13,93 @@ from toot.utils import get_text, parse_html
 from toot.wcstring import wc_wrap
 
 
-START_CODES = {
-    'red': '\033[31m',
-    'green': '\033[32m',
-    'yellow': '\033[33m',
-    'blue': '\033[34m',
-    'magenta': '\033[35m',
-    'cyan': '\033[36m',
+STYLES = {
+    'reset': '\033[0m',
+    'bold': '\033[1m',
+    'dim': '\033[2m',
+    'italic': '\033[3m',
+    'underline': '\033[4m',
+    'red': '\033[91m',
+    'green': '\033[92m',
+    'yellow': '\033[93m',
+    'blue': '\033[94m',
+    'magenta': '\033[95m',
+    'cyan': '\033[96m',
 }
 
-END_CODE = '\033[0m'
-
-START_PATTERN = "<(" + "|".join(START_CODES.keys()) + ")>"
-
-END_PATTERN = "</(" + "|".join(START_CODES.keys()) + ")>"
+STYLE_TAG_PATTERN = re.compile(r"""
+    (?<!\\)     # not preceeded by a backslash - allows escaping
+    <           # literal
+    (/)?        # optional closing - first group
+    (.*?)       # style names - ungreedy - second group
+    >           # literal
+""", re.X)
 
 
-def start_code(match):
-    name = match.group(1)
-    return START_CODES[name]
+def colorize(message):
+    """
+    Replaces style tags in `message` with ANSI escape codes.
+
+    Markup is inspired by HTML, but you can use multiple words pre tag, e.g.:
+
+        <red bold>alert!</red bold> a thing happened
+
+    Empty closing tag will reset all styes:
+
+        <red bold>alert!</> a thing happened
+
+    Styles can be nested:
+
+        <red>red <underline>red and underline</underline> red</red>
+    """
+
+    def _codes(styles):
+        for style in styles:
+            yield STYLES.get(style, "")
+
+    def _generator(message):
+        # A list is used instead of a set because we want to keep style order
+        # This allows nesting colors, e.g. "<blue>foo<red>bar</red>baz</blue>"
+        position = 0
+        active_styles = []
+
+        for match in re.finditer(STYLE_TAG_PATTERN, message):
+            is_closing = bool(match.group(1))
+            styles = match.group(2).strip().split()
+
+            start, end = match.span()
+            # Replace backslash for escaped <
+            yield message[position:start].replace("\\<", "<")
+
+            if is_closing:
+                yield STYLES["reset"]
+
+                # Empty closing tag resets all styles
+                if styles == []:
+                    active_styles = []
+                else:
+                    active_styles = [s for s in active_styles if s not in styles]
+                    yield from _codes(active_styles)
+            else:
+                active_styles = active_styles + styles
+                yield from _codes(styles)
+
+            position = end
+
+        if position == 0:
+            # Nothing matched, yield the original string
+            yield message
+        else:
+            # Yield the remaining fragment
+            yield message[position:]
+            # Reset styles at the end to prevent leaking
+            yield STYLES["reset"]
+
+    return "".join(_generator(message))
 
 
-def colorize(text):
-    text = re.sub(START_PATTERN, start_code, text)
-    text = re.sub(END_PATTERN, END_CODE, text)
-
-    return text
-
-
-def strip_tags(text):
-    text = re.sub(START_PATTERN, '', text)
-    text = re.sub(END_PATTERN, '', text)
-
-    return text
+def strip_tags(message):
+    return re.sub(STYLE_TAG_PATTERN, "", message)
 
 
 def use_ansi_color():
@@ -79,15 +133,15 @@ def print_out(*args, **kwargs):
 
 
 def print_err(*args, **kwargs):
-    args = ["<red>{}</red>".format(a) for a in args]
+    args = [f"<red>{a}</red>" for a in args]
     args = [colorize(a) if USE_ANSI_COLOR else strip_tags(a) for a in args]
     print(*args, file=sys.stderr, **kwargs)
 
 
 def print_instance(instance):
-    print_out("<green>{}</green>".format(instance['title']))
-    print_out("<blue>{}</blue>".format(instance['uri']))
-    print_out("running Mastodon {}".format(instance['version']))
+    print_out(f"<green>{instance['title']}</green>")
+    print_out(f"<blue>{instance['uri']}</blue>")
+    print_out(f"running Mastodon {instance['version']}")
     print_out()
 
     description = instance.get("description")
@@ -113,7 +167,7 @@ def print_instance(instance):
 
 
 def print_account(account):
-    print_out("<green>@{}</green> {}".format(account['acct'], account['display_name']))
+    print_out(f"<green>@{account['acct']}</green> {account['display_name']}")
 
     note = get_text(account['note'])
 
@@ -122,14 +176,14 @@ def print_account(account):
         print_out("\n".join(wrap(note)))
 
     print_out("")
-    print_out("ID: <green>{}</green>".format(account['id']))
-    print_out("Since: <green>{}</green>".format(account['created_at'][:19].replace('T', ' @ ')))
+    print_out(f"ID: <green>{account['id']}</green>")
+    print_out(f"Since: <green>{account['created_at'][:10]}</green>")
     print_out("")
-    print_out("Followers: <yellow>{}</yellow>".format(account['followers_count']))
-    print_out("Following: <yellow>{}</yellow>".format(account['following_count']))
-    print_out("Statuses: <yellow>{}</yellow>".format(account['statuses_count']))
+    print_out(f"Followers: <yellow>{account['followers_count']}</yellow>")
+    print_out(f"Following: <yellow>{account['following_count']}</yellow>")
+    print_out(f"Statuses: <yellow>{account['statuses_count']}</yellow>")
     print_out("")
-    print_out(account['url'])
+    print_out(account["url"])
 
 
 HASHTAG_PATTERN = re.compile(r'(?<!\w)(#\w+)\b')
@@ -141,10 +195,7 @@ def highlight_hashtags(line):
 
 def print_acct_list(accounts):
     for account in accounts:
-        print_out("* <green>@{}</green> {}".format(
-            account['acct'],
-            account['display_name']
-        ))
+        print_out(f"* <green>@{account['acct']}</green> {account['display_name']}")
 
 
 def print_search_results(results):
@@ -157,7 +208,7 @@ def print_search_results(results):
 
     if hashtags:
         print_out("\nHashtags:")
-        print_out(", ".join(["<green>#{}</green>".format(t["name"]) for t in hashtags]))
+        print_out(", ".join([f"<green>#{t['name']}</green>" for t in hashtags]))
 
     if not accounts and not hashtags:
         print_out("<yellow>Nothing found</yellow>")
@@ -174,18 +225,18 @@ def print_status(status, width):
     time = time.strftime('%Y-%m-%d %H:%M %Z')
 
     username = "@" + status['account']['acct']
-    spacing = width - wcswidth(username) - wcswidth(time)
+    spacing = width - wcswidth(username) - wcswidth(time) - 2
 
     display_name = status['account']['display_name']
     if display_name:
         spacing -= wcswidth(display_name) + 1
 
-    print_out("{}{}{}{}".format(
-        "<green>{}</green> ".format(display_name) if display_name else "",
-        "<blue>{}</blue>".format(username),
+    print_out(
+        f"<green>{display_name}</green>" if display_name else "",
+        f"<blue>{username}</blue>",
         " " * spacing,
-        "<yellow>{}</yellow>".format(time),
-    ))
+        f"<yellow>{time}</yellow>",
+    )
 
     for paragraph in parse_html(content):
         print_out("")
@@ -203,11 +254,12 @@ def print_status(status, width):
     if poll:
         print_poll(poll)
 
-    print_out("\n{}{}{}".format(
-        "ID <yellow>{}</yellow>  ".format(status['id']),
-        "↲ In reply to <yellow>{}</yellow>  ".format(in_reply_to) if in_reply_to else "",
-        "↻ Reblogged <blue>@{}</blue>  ".format(reblog['account']['acct']) if reblog else "",
-    ))
+    print_out()
+    print_out(
+        f"ID <yellow>{status['id']}</yellow> ",
+        f"↲ In reply to <yellow>{in_reply_to}</yellow> " if in_reply_to else "",
+        f"↻ Reblogged <blue>@{reblog['account']['acct']}</blue> " if reblog else "",
+    )
 
 
 def print_poll(poll):
@@ -232,7 +284,8 @@ def print_poll(poll):
         expires_at = parse_datetime(poll["expires_at"]).strftime("%Y-%m-%d %H:%M")
         poll_footer += f" · Closes on {expires_at}"
 
-    print_out("\n{}".format(poll_footer))
+    print_out()
+    print_out(poll_footer)
 
 
 def print_timeline(items, width=100):
